/* tsec.h - macros for TSEC1 and TSEC2 controllers */

/*  Copyright 2005-2006 Freescale Semiconductor */
/*
modification history
--------------------
01a,19jul05,ahk written
*/

/*
DESCRIPTION
This module is illustrates how the TSEC controllers (TSEC1/TSEC2) of
MPC8560 can be programmed. It shows how the mac loopback works, how the 
PHY digital loopback works and it also shows how the TSEC controller 
interacts with the external world, that is generating and receiving Ethernet
traffic from external wires. The tested modes are GMII, MII, TBI.  

INCLUDE Files: MPC8560.h
*/

#ifndef __INCtsech
#define __INCtsech
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
	
/* defines */

//#define MAC_INTERNAL_LOOPBACK                  /* loopback test using mac only */
#define PHY_LOOPBACK                         /* enables loopback using the PHY dev */
//#define NORMAL_MODE                            /* transmitting to wires and receiving from wires */

#define INTERFACE_MODE  	GMII_INTERFACE
#define GMII_INTERFACE  	0x00000000
#define MII_INTERFACE	    0x00000001
#define TBI_INTERFACE  		0x00000002

//#define STATUS_POLLING  /* Status of tranmission and reception of */
						  /* frames are done by polling status register */
#define STATUS_INTERRUPT  /* Status of tranmission and reception of */
						  /* frames are done by interrupt mechanism */

#define CCSRBAR_ADDR		0x40000000       /* this start addr of ccsrbar is provided by cfg file */

#define NUM_OF_BD_BUFFER    0x00000020       /* enables loopback using the PHY dev */
#define RX_BUFFER_LOCATION  0x00D00000       /* receive buffer starts at this ddr mem */
                                             /* location, before changing this value */
                                             /* check codewarrior settings so that the */
                                             /* assigned value falls within DDR and it */
                                             /* doesn't conflict with stack and heap */
#define RX_BUFFER_SIZE      0x00000200       /* size per receive buffer */  
#define TX_BUFFER_SIZE      0x00000100       /* size per transmit buffer buffer */  
#define ETHERNET_HDR_SIZE   0x0000000E       /* this contains dst, src addr and length fields */

#define TSEC_ID_1           0x00000000       /* refers to TSEC controller 1 */
#define TSEC_ID_2           0x00000001		 /* refers to TSEC controller 2 */

#define TSEC1_PHY_ADDR      0x00000000       /* check your board and find out whether you */
											 /* have config plug on connector J55; with this */ 
											 /* plug phy addr for TSEC1 is 15 and without that */
											 /* it is 0 */
#define TSEC2_PHY_ADDR      0x00000001       /* check your board and find out whether you */
											 /* have config plug on connector J55; with this */ 
											 /* plug phy addr for TSEC2 is 9 and without that */
											 /* it is 1 */	
#define REG0_MARVELL_PHY    0x00000000											 										 
#define REG1_MARVELL_PHY    0x00000001
#define REG2_MARVELL_PHY    0x00000002
#define REG3_MARVELL_PHY    0x00000003
#define REG5_MARVELL_PHY    0x00000004											 										 
#define REG6_MARVELL_PHY    0x00000005
#define REG7_MARVELL_PHY    0x00000006
#define REG8_MARVELL_PHY    0x00000007

#define REG9_MARVELL_PHY    0x00000009											 										 
#define REG10_MARVELL_PHY   0x0000000A
#define REG11_MARVELL_PHY   0x0000000B
#define REG12_MARVELL_PHY   0x0000000C
#define REG13_MARVELL_PHY   0x0000000D											 										 
#define REG14_MARVELL_PHY   0x0000000E
#define REG15_MARVELL_PHY   0x0000000F
#define REG16_MARVELL_PHY   0x00000010
#define REG17_MARVELL_PHY   0x00000011
#define REG18_MARVELL_PHY   0x00000012

#define REG0_15_RESET_FIELD    		0x00008000
#define REG0_14_LOOPBACK_FIELD 		0x00004000
#define REG0_15_DUPLEX_FIELD	 	0x00000100
#define REG0_15_GIGASPEED_FIELD 	0x00000040



/* typedefs */
typedef _Packed struct
	{
	unsigned short flags;
	unsigned short length;
	unsigned long  Buffer;
	} _PackedType TSEC_TXBD;


typedef _Packed struct
	{
	unsigned short flags;
	unsigned short length;
	unsigned long  Buffer;	
	} _PackedType TSEC_RXBD;

typedef _Packed struct
	{
	unsigned long  DestAddr1;
	unsigned short DestAddr2;
	unsigned long  SourceAddr1;
	unsigned short SourceAddr2;
	unsigned short LengthOrType;
	unsigned char  Data [242];
	} _PackedType TSEC_TXBUFFER;

/* function declarations */
extern mpc_imm                *pCCSRBAR;
extern mpq_tsec				  *pTsecMap;

extern void mgmtInit (void);
extern void mgmtWrite (unsigned long phy_addr, unsigned long reg_addr, unsigned long data);
extern unsigned long mgmtRead (unsigned long phy_addr, unsigned long reg_addr);
extern void tsecInit (void);

extern void GMIIMgmtInit(void);
extern void GMIIMgmtWrite (unsigned long phy_addr, unsigned long reg_addr, unsigned long data);
extern unsigned long GMIIMgmtRead(unsigned long phy_addr, unsigned long reg_addr);
extern void phyInit (void);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INCtsech */